/*
 * Copyright (c) 2010-2022 Belledonne Communications SARL.
 *
 * This file is part of Liblinphone 
 * (see https://gitlab.linphone.org/BC/public/liblinphone).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MSC_VER
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif // _MSC_VER

#ifndef _ACCOUNT_DEVICE_HH
#define _ACCOUNT_DEVICE_HH

#include <string>
#include "object.hh"




struct _LinphoneAccountDevice;


namespace linphone {


	/**
	 * @brief Object that represents a device that at least once was connected to a
	 * given account. 
	 *
	 */
	class AccountDevice: public Object {
	
		public:

			LINPHONECXX_PUBLIC AccountDevice(void *ptr, bool takeRef=true);
			LINPHONECXX_PUBLIC _LinphoneAccountDevice *cPtr() {return (_LinphoneAccountDevice *)mPrivPtr;}

	
		public:
			
		
			/**
			 * @brief Gets the timestamp at which this devices was updated for the last time. 
			 *
			 * @return the time under ISO 8601 format at which the device was updated.    
			 */
			LINPHONECXX_PUBLIC std::string getLastUpdateTime() const;
			
			/**
			 * @brief Gets the timestamp at which this devices was updated for the last time. 
			 *
			 * @return the timestamp (time_t) at which the device was updated.    
			 */
			LINPHONECXX_PUBLIC time_t getLastUpdateTimestamp() const;
			
			/**
			 * @brief Gets the name of the device. 
			 *
			 * @return the name of the device.    
			 */
			LINPHONECXX_PUBLIC std::string getName() const;
			
			/**
			 * @brief Gets the user-agent of the device. 
			 *
			 * @return the user-agent of the device.    
			 */
			LINPHONECXX_PUBLIC std::string getUserAgent() const;
			
			/**
			 * @brief Gets the UUID of the device. 
			 *
			 * @return the UUID of the device.    
			 */
			LINPHONECXX_PUBLIC std::string getUuid() const;
			
			
	};

};

#endif // _ACCOUNT_DEVICE_HH

#ifndef _MSC_VER
#pragma GCC diagnostic pop
#endif // _MSC_VER
